@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyPromptService.cpp,v 1.9 2006/03/05 20:54:32 chpe Exp $
+ *  $Id: EphyPromptService.cpp,v 1.12 2006/04/01 11:56:36 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -68,8 +68,6 @@ enum
 	RESPONSE_ABORT_SCRIPT = 42
 };
 
-#define RETVAL(r) ((r) == GTK_RESPONSE_DELETE_EVENT ? NS_ERROR_FAILURE : NS_OK)
-
 class Prompter
 {
 public:
@@ -135,8 +133,7 @@ Prompter::Prompter (const char *aStock,
 	mEntries[0] = mEntries[1] = nsnull;
 
 	mDialog = GTK_DIALOG (gtk_dialog_new ());
-	g_object_ref (mDialog);
-	gtk_object_sink (GTK_OBJECT (mDialog));
+	g_object_ref_sink (mDialog);
 
 	char *title = NULL;
 	if (aTitle)
@@ -457,11 +454,10 @@ EnableResponse (GtkDialog *aDialog)
 }
 
 static void
-RemoveTimeout (GObject *aDialog)
+RemoveTimeout (gpointer idptr)
 {
-	guint timeout;
+	guint timeout = GPOINTER_TO_UINT (idptr);
 
-	timeout = GPOINTER_TO_UINT (g_object_get_data (aDialog, TIMEOUT_DATA_KEY));
 	g_return_if_fail (timeout != 0);
 
 	g_source_remove (timeout);
@@ -663,9 +659,9 @@ EphyPromptService::Alert (nsIDOMWindow *aParent,
 {
 	Prompter prompt (GTK_STOCK_DIALOG_INFO, aParent, aDialogTitle, aText);
 	prompt.AddStockButton (GTK_STOCK_OK, GTK_RESPONSE_ACCEPT);
-	PRInt32 response = prompt.Run ();
+	prompt.Run ();
 
-	return RETVAL(response);
+	return NS_OK;
 }
 
 /* void alertCheck (in nsIDOMWindow aParent, in wstring aDialogTitle, in wstring aText, in wstring aCheckMsg, inout boolean aCheckState); */
@@ -680,10 +676,10 @@ EphyPromptService::AlertCheck (nsIDOMWindow *aParent,
 	prompt.AddStockButton (GTK_STOCK_OK, GTK_RESPONSE_ACCEPT);
 	prompt.AddCheckbox (aCheckMsg, aCheckState);
 
-	PRInt32 response = prompt.Run ();
+	prompt.Run ();
 	prompt.GetCheckboxState (aCheckState);
 
-	return RETVAL(response);
+	return NS_OK;
 }
 
 /* boolean confirm (in nsIDOMWindow aParent, in wstring aDialogTitle, in wstring aText); */
@@ -698,9 +694,9 @@ EphyPromptService::Confirm (nsIDOMWindow *aParent,
 	Prompter prompt (GTK_STOCK_DIALOG_QUESTION, aParent, aDialogTitle, aText);
 	prompt.AddStockButton (GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 	prompt.AddStockButton (GTK_STOCK_OK, GTK_RESPONSE_ACCEPT);
-	PRInt32 response = prompt.Run (_retval);
+	prompt.Run (_retval);
 
-	return RETVAL(response);
+	return NS_OK;
 }
 
 /* boolean confirmCheck (in nsIDOMWindow aParent, in wstring aDialogTitle, in wstring aText, in wstring aCheckMsg, inout boolean aCheckState); */
@@ -719,10 +715,10 @@ EphyPromptService::ConfirmCheck (nsIDOMWindow *aParent,
 	prompt.AddStockButton (GTK_STOCK_OK, GTK_RESPONSE_ACCEPT);
 	prompt.AddCheckbox (aCheckMsg, aCheckState);
 
-	PRInt32 response = prompt.Run (_retval);
+	prompt.Run (_retval);
 	prompt.GetCheckboxState (aCheckState);
 
-	return RETVAL(response);
+	return NS_OK;
 }
 
 /* PRInt32 confirmEx (in nsIDOMWindow aParent, in wstring aDialogTitle, in wstring aText, in unsigned long aButtonFlags, in wstring aButton0Title, in wstring aButton1Title, in wstring aButton2Title, in wstring aCheckMsg, inout boolean aCheckState); */
@@ -745,11 +741,10 @@ EphyPromptService::ConfirmEx (nsIDOMWindow *aParent,
 				    aButton1Title, aButton2Title);
 	prompt.AddCheckbox (aCheckMsg, aCheckState);
 
-	PRInt32 response = prompt.Run (nsnull);
-	*_retval = response;
+	*_retval = prompt.Run (nsnull);
 	prompt.GetCheckboxState (aCheckState);
 
-	return RETVAL(response);
+	return NS_OK;
 }
 
 /* boolean prompt (in nsIDOMWindow aParent, in wstring aDialogTitle, in wstring aText, inout wstring aValue, in wstring aCheckMsg, inout boolean aCheckState); */
@@ -771,11 +766,11 @@ EphyPromptService::Prompt (nsIDOMWindow *aParent,
 	prompt.AddEntry (nsnull, *aValue, PR_FALSE);
 	prompt.AddCheckbox (aCheckMsg, aCheckState);
 
-	PRInt32 response = prompt.Run (_retval);
+	prompt.Run (_retval);
 	prompt.GetText (0, aValue);
 	prompt.GetCheckboxState (aCheckState);
 
-	return RETVAL(response);
+	return NS_OK;
 }
 
 /* boolean promptUsernameAndPassword (in nsIDOMWindow aParent, in wstring aDialogTitle, in wstring aText, inout wstring aUsername, inout wstring aPassword, in wstring aCheckMsg, inout boolean aCheckState); */
@@ -800,12 +795,12 @@ EphyPromptService::PromptUsernameAndPassword (nsIDOMWindow *aParent,
 	prompt.AddEntry (_("_Password:"), *aPassword, PR_TRUE);
 	prompt.AddCheckbox (aCheckMsg, aCheckState);
 
-	PRInt32 response = prompt.Run (_retval);
+	prompt.Run (_retval);
 	prompt.GetText (0, aUsername);
 	prompt.GetText (1, aPassword);
 	prompt.GetCheckboxState (aCheckState);
 
-	return RETVAL(response);
+	return NS_OK;
 }
 
 /* boolean promptPassword (in nsIDOMWindow aParent, in wstring aDialogTitle, in wstring aText, inout wstring aPassword, in wstring aCheckMsg, inout boolean aCheckState); */
@@ -829,11 +824,11 @@ EphyPromptService::PromptPassword (nsIDOMWindow *aParent,
 
 	// FIXME: Add a CAPSLOCK indicator?
 
-	PRInt32 response = prompt.Run (_retval);
+	prompt.Run (_retval);
 	prompt.GetText (0, aPassword);
 	prompt.GetCheckboxState (aCheckState);
 
-	return RETVAL(response);
+	return NS_OK;
 }
 
 /* boolean select (in nsIDOMWindow aParent, in wstring aDialogTitle, in wstring aText, in PRUint32 aCount, [array, size_is (aCount)] in wstring aSelectList, out long aOutSelection); */
@@ -854,10 +849,10 @@ EphyPromptService::Select (nsIDOMWindow *aParent,
 	prompt.AddStockButton (GTK_STOCK_OK, GTK_RESPONSE_ACCEPT);
 	prompt.AddSelect (aCount, aSelectList, *aOutSelection);
 
-	PRInt32 response = prompt.Run (_retval);
+	prompt.Run (_retval);
 	prompt.GetSelected (aOutSelection);
 
-	return RETVAL(response);
+	return NS_OK;
 }
 
 #if HAVE_NSINONBLOCKINGALERTSERVICE_H
