@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: GlobalHistory.cpp,v 1.43 2006/03/03 17:28:31 chpe Exp $
+ *  $Id: GlobalHistory.cpp,v 1.44 2006/03/07 22:34:21 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -161,6 +161,9 @@ NS_IMETHODIMP MozGlobalHistory::SetPageTitle(nsIURI *aURI,
 }
 
 #ifdef HAVE_NSIGLOBALHISTORY3_H
+
+#ifdef HAVE_GECKO_1_9
+
 /* unsigned long getURIGeckoFlags(in nsIURI aURI); */
 NS_IMETHODIMP
 MozGlobalHistory::GetURIGeckoFlags(nsIURI *aURI,
@@ -214,6 +217,8 @@ MozGlobalHistory::SetURIGeckoFlags(nsIURI *aURI,
 	return NS_ERROR_FAILURE;
 }
 
+#endif /* HAVE_GECKO_1_9 */
+
 /* void addDocumentRedirect (in nsIChannel 
    		             aOldChannel, 
 			     in nsIChannel aNewChannel, 
