@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: MozRegisterComponents.cpp,v 1.47 2006/03/05 22:07:52 chpe Exp $
+ *  $Id: MozRegisterComponents.cpp,v 1.48 2006/03/19 20:48:13 jframeau Exp $
  */
 
 #include "mozilla-config.h"
@@ -54,6 +54,10 @@
 #include <nsILocalFile.h>
 #include <nsNetCID.h>
 
+#ifdef HAVE_GECKO_1_9
+#include <nsIClassInfoImpl.h>
+#endif
+
 #include <glib/gmessages.h>
 
 NS_GENERIC_FACTORY_CONSTRUCTOR(MozDownload)
