@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed-single.cpp,v 1.138.2.1 2006/05/15 18:32:49 chpe Exp $
+ *  $Id: mozilla-embed-single.cpp,v 1.138 2006/03/05 20:54:32 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -296,14 +296,14 @@ mozilla_set_default_prefs (MozillaEmbedSingle *mes)
         }
 
 #ifdef HAVE_GECKO_1_8
-	pref->SetCharPref ("general.useragent.extra.epiphany", "Epiphany/" EPIPHANY_MAJOR);
+	pref->SetCharPref ("general.useragent.extra.epiphany", "Epiphany/" UA_VERSION);
 
 	/* Unset old prefs, otherwise they end up in the user agent string too */
 	pref->ClearUserPref ("general.useragent.vendor");
 	pref->ClearUserPref ("general.useragent.vendorSub");
 #else
 	pref->SetCharPref ("general.useragent.vendor", "Epiphany");
-	pref->SetCharPref ("general.useragent.vendorSub", EPIPHANY_MAJOR);
+	pref->SetCharPref ("general.useragent.vendorSub", UA_VERSION);
 #endif
 
 	/* Don't open ftp uris with an external handler if one is setup */
@@ -421,7 +421,7 @@ mozilla_init_plugin_path ()
 #ifdef HAVE_PRIVATE_PLUGINS
 				":" PLUGINDIR,
 #endif
-				(char *) NULL);
+				NULL);
 
 	g_setenv ("MOZ_PLUGIN_PATH", new_path, TRUE);
 	g_free (new_path);
@@ -451,7 +451,7 @@ mozilla_init_profile (void)
 	char *profile_path;
 	profile_path = g_build_filename (ephy_dot_dir (), 
 					 MOZILLA_PROFILE_DIR,
-					 (char *) NULL);
+					 NULL);
         gtk_moz_embed_set_profile_path (profile_path, MOZILLA_PROFILE_NAME);
         g_free (profile_path);
 }
@@ -676,7 +676,7 @@ mozilla_stylesheet_init (MozillaEmbedSingle *single)
 
 	priv->user_css_file = g_build_filename (ephy_dot_dir (),
 						USER_STYLESHEET_FILENAME,
-						(char *) NULL);
+						NULL);
 
 	user_css_enabled_notify (NULL, 0, NULL, single);
 	priv->user_css_enabled_notifier_id =
@@ -801,7 +801,7 @@ mozilla_embed_single_init (MozillaEmbedSingle *mes)
 				  MOZILLA_PROFILE_DIR,
 				  MOZILLA_PROFILE_NAME,
 				  MOZILLA_PROFILE_FILE,
-				  (char *) NULL);
+				  NULL);
 
 	g_signal_connect_object (embed_shell, "prepare-close",
 				 G_CALLBACK (prepare_close_cb), mes,
