@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  $Id: ephy-find-toolbar.c,v 1.26 2006/01/10 12:50:56 paharvey Exp $
+ *  $Id: ephy-find-toolbar.c,v 1.28 2006/04/10 20:00:17 chpe Exp $
  */
 
 #include "config.h"
@@ -101,17 +101,6 @@ get_find (EphyFindToolbar *toolbar)
 	return priv->find;
 }
 
-static void
-set_controls (EphyFindToolbar *toolbar,
-	      gboolean can_find_next,
-	      gboolean can_find_prev)
-{
-	EphyFindToolbarPrivate *priv = toolbar->priv;
-
-	gtk_widget_set_sensitive (GTK_WIDGET (priv->next), can_find_next);
-	gtk_widget_set_sensitive (GTK_WIDGET (priv->prev), can_find_prev);
-}
-
 static void
 set_status (EphyFindToolbar *toolbar,
 	    EphyEmbedFindResult result)
@@ -151,14 +140,6 @@ clear_status (EphyFindToolbar *toolbar)
 			    priv->links_only ? _("Find links:") : _("Find:"));
 }
 
-static void
-tab_content_changed_cb (EphyEmbed *embed,
-			const char *uri,
-			EphyFindToolbar *toolbar)
-{
-	set_controls (toolbar, TRUE, TRUE);
-}
-
 #ifdef HAVE_TYPEAHEADFIND
 
 /* Code adapted from gtktreeview.c:gtk_tree_view_key_press() and
@@ -219,7 +200,7 @@ entry_changed_cb (GtkEntry *entry,
 #ifdef HAVE_TYPEAHEADFIND
 	EphyEmbedFindResult result;
 #endif
-	gboolean found = TRUE, case_sensitive;
+	gboolean case_sensitive;
 
 	text = gtk_entry_get_text (GTK_ENTRY (priv->entry));
 
@@ -234,10 +215,8 @@ entry_changed_cb (GtkEntry *entry,
 #ifdef HAVE_TYPEAHEADFIND
 	result = ephy_embed_find_find (get_find (toolbar), text, priv->links_only);
 
-	found = result != EPHY_EMBED_FIND_NOTFOUND;
 	set_status (toolbar, result);
 #endif
-	set_controls (toolbar, found, found);
 }
 
 static gboolean
@@ -440,7 +419,7 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 }
 
 static void
-ephy_find_toolbar_finalize (GObject *object)
+ephy_find_toolbar_dispose (GObject *object)
 {
 	EphyFindToolbar *toolbar = EPHY_FIND_TOOLBAR (object);
 	EphyFindToolbarPrivate *priv = toolbar->priv;
@@ -448,9 +427,10 @@ ephy_find_toolbar_finalize (GObject *object)
 	if (priv->find != NULL)
 	{
 		g_object_unref (priv->find);
+		priv->find = NULL;
 	}
 
-	parent_class->finalize (object);
+	parent_class->dispose (object);
 }
 
 static void
@@ -487,7 +467,7 @@ ephy_find_toolbar_class_init (EphyFindToolbarClass *klass)
 
         parent_class = g_type_class_peek_parent (klass);
 
-	object_class->finalize = ephy_find_toolbar_finalize;
+	object_class->dispose = ephy_find_toolbar_dispose;
 	object_class->get_property = ephy_find_toolbar_get_property;
 	object_class->set_property = ephy_find_toolbar_set_property;
 
@@ -599,12 +579,8 @@ ephy_find_toolbar_set_embed (EphyFindToolbar *toolbar,
 	priv->embed = embed;
 	if (embed != NULL)
 	{
-		set_controls (toolbar, TRUE, TRUE);
 		clear_status (toolbar);
 
-		g_signal_connect_object (embed, "ge-content-change",
-					 G_CALLBACK (tab_content_changed_cb),
-					 toolbar, G_CONNECT_AFTER);
 #ifdef HAVE_TYPEAHEADFIND
 		g_signal_connect_object (embed, "ge-search-key-press",
 					 G_CALLBACK (tab_search_key_press_cb),
@@ -624,12 +600,9 @@ void
 ephy_find_toolbar_find_next (EphyFindToolbar *toolbar)
 {
 	EphyEmbedFindResult result;
-	gboolean found;
 
 	result = ephy_embed_find_find_again (get_find (toolbar), TRUE);
 
-	found = result != EPHY_EMBED_FIND_NOTFOUND;
-	set_controls (toolbar, found, found);
 	set_status (toolbar, result);
 }
 
@@ -637,11 +610,9 @@ void
 ephy_find_toolbar_find_previous (EphyFindToolbar *toolbar)
 {
 	EphyEmbedFindResult result;
-	gboolean found;
 
 	result = ephy_embed_find_find_again (get_find (toolbar), FALSE);
-	found = result != EPHY_EMBED_FIND_NOTFOUND;
-	set_controls (toolbar, found, found);
+
 	set_status (toolbar, result);
 }
 
