@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-fullscreen-popup.c,v 1.8 2005/11/08 12:56:23 chpe Exp $
+ *  $Id: ephy-fullscreen-popup.c,v 1.9 2006/03/28 20:06:55 chpe Exp $
  */
 
 #include "config.h"
@@ -239,8 +239,7 @@ ephy_fullscreen_popup_constructor (GType type,
 	gtk_window_set_resizable (window, FALSE);
 
 	priv->tooltips = gtk_tooltips_new ();
-	g_object_ref (G_OBJECT (priv->tooltips));
-	gtk_object_sink (GTK_OBJECT (priv->tooltips));
+	g_object_ref_sink (priv->tooltips);
 
 	hbox = gtk_hbox_new (FALSE, 2);
 	gtk_container_add (GTK_CONTAINER (window), hbox);
