@@ -1,7 +1,7 @@
 /*
  *  Copyright (C) 2002 Christophe Fergeau
  *  Copyright (C) 2003, 2004 Marco Pesenti Gritti
- *  Copyright (C) 2003, 2004 Christian Persch
+ *  Copyright (C) 2003, 2004, 2005 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,13 +17,12 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-notebook.c,v 1.75 2005/07/03 21:38:55 jframeau Exp $
+ *  $Id: ephy-notebook.c,v 1.89 2006/03/28 20:06:55 chpe Exp $
  */
 
 #include "config.h"
 
 #include "ephy-notebook.h"
-#include "ephy-stock-icons.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
 #include "ephy-marshal.h"
@@ -32,25 +31,21 @@
 #include "ephy-embed.h"
 #include "ephy-window.h"
 #include "ephy-shell.h"
-#include "ephy-favicon-cache.h"
 #include "ephy-spinner.h"
 #include "ephy-link.h"
 #include "ephy-debug.h"
 
-#include <glib-object.h>
+#include <glib/gi18n.h>
 #include <gtk/gtkeventbox.h>
-#include <gtk/gtknotebook.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkwidget.h>
 #include <gtk/gtktooltips.h>
-#include <gtk/gtkmain.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkbutton.h>
+#include <gtk/gtkaccelgroup.h>
 #include <gtk/gtkiconfactory.h>
-#include <glib/gi18n.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
 
 #define TAB_WIDTH_N_CHARS 15
 
@@ -69,11 +64,7 @@ struct _EphyNotebookPrivate
 	GList *focused_pages;
 	GtkTooltips *title_tips;
 	guint tabs_vis_notifier_id;
-	gulong motion_notify_handler_id;
-	int x_start;
-	int y_start;
 
-	guint drag_in_progress : 1;
 	guint show_tabs : 1;
 	guint dnd_enabled : 1;
 };
@@ -81,18 +72,18 @@ struct _EphyNotebookPrivate
 static void ephy_notebook_init           (EphyNotebook *notebook);
 static void ephy_notebook_class_init     (EphyNotebookClass *klass);
 static void ephy_notebook_finalize       (GObject *object);
-static void move_tab_to_another_notebook (EphyNotebook *src,
-			                  EphyNotebook *dest,
-					  GdkEventMotion *event,
-					  int dest_position);
-
-/* Local variables */
-static GdkCursor *cursor = NULL;
+static int  ephy_notebook_insert_page	 (GtkNotebook *notebook,
+					  GtkWidget *child,
+					  GtkWidget *tab_label,
+					  GtkWidget *menu_label,
+					  int position);
+static void ephy_notebook_remove	 (GtkContainer *container,
+					  GtkWidget *tab_widget);
 
 static const GtkTargetEntry url_drag_types [] = 
 {
-        { EPHY_DND_URI_LIST_TYPE,   0, 0 },
-        { EPHY_DND_URL_TYPE,        0, 1 }
+	{ EPHY_DND_URI_LIST_TYPE,   0, 0 },
+	{ EPHY_DND_URL_TYPE,        0, 1 }
 };
 
 enum
@@ -104,17 +95,12 @@ enum
 
 enum
 {
-	TAB_ADDED,
-	TAB_REMOVED,
-	TABS_REORDERED,
-	TAB_DETACHED,
-	TAB_DELETE,
+	TAB_CLOSE_REQUEST,
 	LAST_SIGNAL
 };
 
-static guint signals[LAST_SIGNAL] = { 0 };
-
-static GObjectClass *parent_class = NULL;
+static guint signals[LAST_SIGNAL];
+static GObjectClass *parent_class;
 
 GType
 ephy_notebook_get_type (void)
@@ -209,6 +195,8 @@ static void
 ephy_notebook_class_init (EphyNotebookClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkContainerClass *container_class = GTK_CONTAINER_CLASS (klass);
+	GtkNotebookClass *notebook_class = GTK_NOTEBOOK_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
@@ -216,53 +204,18 @@ ephy_notebook_class_init (EphyNotebookClass *klass)
 	object_class->get_property = ephy_notebook_get_property;
 	object_class->set_property = ephy_notebook_set_property;
 
-	signals[TAB_ADDED] =
-		g_signal_new ("tab_added",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (EphyNotebookClass, tab_added),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__OBJECT,
-			      G_TYPE_NONE,
-			      1,
-			      EPHY_TYPE_TAB);
-	signals[TAB_REMOVED] =
-		g_signal_new ("tab_removed",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (EphyNotebookClass, tab_removed),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__OBJECT,
-			      G_TYPE_NONE,
-			      1,
-			      EPHY_TYPE_TAB);
-	signals[TAB_DETACHED] =
-		g_signal_new ("tab_detached",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (EphyNotebookClass, tab_detached),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__OBJECT,
-			      G_TYPE_NONE,
-			      1,
-			      EPHY_TYPE_TAB);
-	signals[TABS_REORDERED] =
-		g_signal_new ("tabs_reordered",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (EphyNotebookClass, tabs_reordered),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__VOID,
-			      G_TYPE_NONE,
-			      0);
-	signals[TAB_DELETE] =
-		g_signal_new ("tab_delete",
+	container_class->remove = ephy_notebook_remove;
+
+	notebook_class->insert_page = ephy_notebook_insert_page;
+
+	signals[TAB_CLOSE_REQUEST] =
+		g_signal_new ("tab-close-request",
 			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (EphyNotebookClass, tab_delete),
-			      g_signal_accumulator_true_handled, NULL,
-			      ephy_marshal_BOOLEAN__OBJECT,
-			      G_TYPE_BOOLEAN,
+			      G_STRUCT_OFFSET (EphyNotebookClass, tab_close_req),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE,
 			      1,
 			      EPHY_TYPE_TAB);
 
@@ -272,7 +225,7 @@ ephy_notebook_class_init (EphyNotebookClass *klass)
 							       "DND enabled",
 							       "Whether the notebook allows tab reordering by DND",
 							       TRUE,
-							       G_PARAM_READWRITE));
+							       G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_object_class_install_property (object_class,
 					 PROP_SHOW_TABS,
@@ -280,11 +233,13 @@ ephy_notebook_class_init (EphyNotebookClass *klass)
 							       "Show tabs",
 							       "Whether the notebook shows the tabs bar",
 							       TRUE,
-							       G_PARAM_READWRITE));
+							       G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_type_class_add_private (object_class, sizeof (EphyNotebookPrivate));
 }
 
+
+/* FIXME remove when gtknotebook's func for this becomes public, bug #.... */
 static EphyNotebook *
 find_notebook_at_pointer (gint abs_x, gint abs_y)
 {
@@ -387,247 +342,6 @@ find_tab_num_at_pos (EphyNotebook *notebook, gint abs_x, gint abs_y)
 	return AFTER_ALL_TABS;
 }
 
-static gint find_notebook_and_tab_at_pos (gint abs_x, gint abs_y,
-					  EphyNotebook **notebook,
-					  gint *page_num)
-{
-	*notebook = find_notebook_at_pointer (abs_x, abs_y);
-	if (*notebook == NULL)
-	{
-		return NOT_IN_APP_WINDOWS;
-	}
-	*page_num = find_tab_num_at_pos (*notebook, abs_x, abs_y);
-
-	if (*page_num < 0)
-	{
-		return *page_num;
-	}
-	else
-	{
-		return 0;
-	}
-}
-
-void
-ephy_notebook_move_tab (EphyNotebook *src,
-			EphyNotebook *dest,
-			EphyTab *tab,
-			int dest_position)
-{
-	if (dest == NULL || src == dest)
-	{
-		gtk_notebook_reorder_child
-			(GTK_NOTEBOOK (src), GTK_WIDGET (tab), dest_position);
-		
-		if (src->priv->drag_in_progress == FALSE)
-		{
-			g_signal_emit (G_OBJECT (src), signals[TABS_REORDERED], 0);
-		}
-	}
-#ifndef KEEP_TAB_IN_SAME_TOPLEVEL
-	else
-	{
-		/* make sure the tab isn't destroyed while we move it */
-		g_object_ref (tab);
-		ephy_notebook_remove_tab (src, tab);
-		ephy_notebook_add_tab (dest, tab, dest_position, TRUE);
-		g_object_unref (tab);
-	}
-#endif
-}
-
-static void
-drag_start (EphyNotebook *notebook,
-	    guint32 time)
-{
-	notebook->priv->drag_in_progress = TRUE;
-
-	/* get a new cursor, if necessary */
-	/* FIXME multi-head */
-	if (!cursor) cursor = gdk_cursor_new (GDK_FLEUR);
-
-	/* grab the pointer */
-	gtk_grab_add (GTK_WIDGET (notebook));
-	/* FIXME multi-head */
-	if (!gdk_pointer_is_grabbed ())
-	{
-		gdk_pointer_grab (GTK_WIDGET (notebook)->window,
-				  FALSE,
-				  GDK_BUTTON1_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
-				  NULL, cursor, time);
-	}
-}
-
-static void
-drag_stop (EphyNotebook *notebook)
-{
-	if (notebook->priv->drag_in_progress)
-	{
-		g_signal_emit (G_OBJECT (notebook), signals[TABS_REORDERED], 0);
-	}
-
-	notebook->priv->drag_in_progress = FALSE;
-	if (notebook->priv->motion_notify_handler_id != 0)
-	{
-		g_signal_handler_disconnect (G_OBJECT (notebook),
-					     notebook->priv->motion_notify_handler_id);
-		notebook->priv->motion_notify_handler_id = 0;
-	}
-}
-
-/* this function is only called during dnd, we don't need to emit TABS_REORDERED
- * here, instead we do it on drag_stop
- */
-static void
-move_tab (EphyNotebook *notebook,
-	  int dest_position)
-{
-	gint cur_page_num;
-
-	cur_page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
-
-	if (dest_position != cur_page_num)
-	{
-		GtkWidget *cur_tab;
-		
-		cur_tab = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-						     cur_page_num);
-		ephy_notebook_move_tab (EPHY_NOTEBOOK (notebook), NULL,
-					EPHY_TAB (cur_tab),
-					dest_position);
-	}
-}
-
-static gboolean
-motion_notify_cb (EphyNotebook *notebook,
-		  GdkEventMotion *event,
-		  gpointer data)
-{
-	EphyNotebook *dest;
-	gint page_num;
-	gint result;
-
-	if (notebook->priv->drag_in_progress == FALSE)
-	{
-		if (gtk_drag_check_threshold (GTK_WIDGET (notebook),
-					      notebook->priv->x_start,
-					      notebook->priv->y_start,
-					      event->x_root, event->y_root))
-		{
-			drag_start (notebook, event->time);
-			return TRUE;
-		}
-
-		return FALSE;
-	}
-
-	result = find_notebook_and_tab_at_pos ((gint)event->x_root,
-					       (gint)event->y_root,
-					       &dest, &page_num);
-
-	if (result != NOT_IN_APP_WINDOWS)
-	{
-		if (dest != notebook)
-		{
-			move_tab_to_another_notebook (notebook, dest,
-						      event, page_num);
-		}
-		else
-		{
-			g_assert (page_num >= -1);
-			move_tab (notebook, page_num);
-		}
-	}
-
-	return FALSE;
-}
-
-static void
-move_tab_to_another_notebook (EphyNotebook *src,
-			      EphyNotebook *dest,
-			      GdkEventMotion *event,
-			      int dest_position)
-{
-#ifndef KEEP_TAB_IN_SAME_TOPLEVEL
-	EphyTab *tab;
-	int cur_page;
-
-	/* This is getting tricky, the tab was dragged in a notebook
-	 * in another window of the same app, we move the tab
-	 * to that new notebook, and let this notebook handle the
-	 * drag
-	*/
-	g_assert (EPHY_IS_NOTEBOOK (dest));
-	g_assert (dest != src);
-
-	/* Check if the dest notebook can accept the drag */
-	if (!dest->priv->dnd_enabled) return;
-
-	cur_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (src));
-	tab = EPHY_TAB (gtk_notebook_get_nth_page (GTK_NOTEBOOK (src), cur_page));
-
-	/* stop drag in origin window */
-	/* ungrab the pointer if it's grabbed */
-	drag_stop (src);
-	if (gdk_pointer_is_grabbed ())
-	{
-		gdk_pointer_ungrab (event->time);
-	}
-	gtk_grab_remove (GTK_WIDGET (src));
-
-	ephy_notebook_move_tab (src, dest, tab, dest_position);
-
-	/* start drag handling in dest notebook */
-
-	dest->priv->motion_notify_handler_id =
-		g_signal_connect (G_OBJECT (dest),
-				  "motion-notify-event",
-				  G_CALLBACK (motion_notify_cb),
-				  NULL);
-
-	drag_start (dest, event->time);
-#endif /* KEEP_TAB_IN_SAME_TOPLEVEL */
-}
-
-static gboolean
-button_release_cb (EphyNotebook *notebook,
-		   GdkEventButton *event,
-		   gpointer data)
-{
-	if (notebook->priv->drag_in_progress)
-	{
-		gint cur_page_num;
-		GtkWidget *cur_page;
-
-		cur_page_num =
-			gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
-		cur_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-						      cur_page_num);
-
-#ifndef KEEP_TAB_IN_SAME_TOPLEVEL
-		if (!is_in_notebook_window (notebook, event->x_root, event->y_root)
-		    && gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook)) > 1)
-		{
-			/* Tab was detached */
-			g_signal_emit (G_OBJECT (notebook),
-				       signals[TAB_DETACHED], 0, cur_page);
-		}
-#endif
-
-		/* ungrab the pointer if it's grabbed */
-		if (gdk_pointer_is_grabbed ())
-		{
-			gdk_pointer_ungrab (event->time);
-		}
-		gtk_grab_remove (GTK_WIDGET (notebook));
-	}
-
-	/* This must be called even if a drag isn't happening */
-	drag_stop (notebook);
-
-	return FALSE;
-}
-
 static gboolean
 button_press_cb (EphyNotebook *notebook,
 		 GdkEventButton *event,
@@ -635,27 +349,11 @@ button_press_cb (EphyNotebook *notebook,
 {
 	int tab_clicked;
 
-	if (!notebook->priv->dnd_enabled) return FALSE;
-
 	tab_clicked = find_tab_num_at_pos (notebook, event->x_root, event->y_root);
 
-	/* FIXME: how could there be a drag in progress!? */
-	if (notebook->priv->drag_in_progress)
-	{
-		return TRUE;
-	}
-
-	if ((event->button == 1) && (event->type == GDK_BUTTON_PRESS)
-	    && (tab_clicked >= 0))
-	{
-		notebook->priv->x_start = event->x_root;
-		notebook->priv->y_start = event->y_root;
-		notebook->priv->motion_notify_handler_id =
-			g_signal_connect (G_OBJECT (notebook),
-					  "motion-notify-event",
-					  G_CALLBACK (motion_notify_cb), NULL);
-	}
-	else if (GDK_BUTTON_PRESS == event->type && 3 == event->button)
+	if (event->type == GDK_BUTTON_PRESS &&
+	    event->button == 3 &&
+		   (event->state & gtk_accelerator_get_default_mod_mask ()) == 0)
 	{
 		if (tab_clicked == -1)
 		{
@@ -664,27 +362,19 @@ button_press_cb (EphyNotebook *notebook,
 			 */
 			return TRUE;
 		}
-		else
-		{
-			/* switch to the page the mouse is over, but don't consume the event */
-			gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), tab_clicked);
-		}
+
+		/* switch to the page the mouse is over, but don't consume the event */
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), tab_clicked);
 	}
 
 	return FALSE;
 }
 
-GtkWidget *
-ephy_notebook_new (void)
-{
-	return GTK_WIDGET (g_object_new (EPHY_TYPE_NOTEBOOK, NULL));
-}
-
 static void
 ephy_notebook_switch_page_cb (GtkNotebook *notebook,
-                              GtkNotebookPage *page,
-                              guint page_num,
-                              gpointer data)
+			      GtkNotebookPage *page,
+			      guint page_num,
+			      gpointer data)
 {
 	EphyNotebook *nb = EPHY_NOTEBOOK (notebook);
 	GtkWidget *child;
@@ -725,7 +415,7 @@ notebook_drag_data_received_cb (GtkWidget* widget, GdkDragContext *context,
 		char **split;
 
 		/* URL_TYPE has format: url \n title */
-		split = g_strsplit ((const gchar *)selection_data->data, "\n", 1);
+		split = g_strsplit ((const gchar *)selection_data->data, "\n", 2);
 		if (split != NULL && split[0] != NULL && split[0][0] != '\0')
 		{
 			ephy_link_open (EPHY_LINK (notebook), split[0], tab,
@@ -794,30 +484,26 @@ ephy_notebook_init (EphyNotebook *notebook)
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
 
 	notebook->priv->title_tips = gtk_tooltips_new ();
-	g_object_ref (G_OBJECT (notebook->priv->title_tips));
-	gtk_object_sink (GTK_OBJECT (notebook->priv->title_tips));
+	g_object_ref_sink (notebook->priv->title_tips);
 
 	notebook->priv->show_tabs = TRUE;
 	notebook->priv->dnd_enabled = TRUE;
 
 	g_signal_connect (notebook, "button-press-event",
 			  (GCallback)button_press_cb, NULL);
-	g_signal_connect (notebook, "button-release-event",
-			  (GCallback)button_release_cb, NULL);
-	gtk_widget_add_events (GTK_WIDGET (notebook), GDK_BUTTON1_MOTION_MASK);
-
-	g_signal_connect_after (G_OBJECT (notebook), "switch_page",
-                                G_CALLBACK (ephy_notebook_switch_page_cb),
-                                NULL);
+	g_signal_connect_after (notebook, "switch-page",
+				G_CALLBACK (ephy_notebook_switch_page_cb),
+				NULL);
 
 	/* Set up drag-and-drop target */
-	g_signal_connect (G_OBJECT(notebook), "drag_data_received",
-			  G_CALLBACK(notebook_drag_data_received_cb),
+	g_signal_connect (notebook, "drag-data-received",
+			  G_CALLBACK (notebook_drag_data_received_cb),
 			  NULL);
-        gtk_drag_dest_set (GTK_WIDGET(notebook), GTK_DEST_DEFAULT_MOTION |
+	gtk_drag_dest_set (GTK_WIDGET (notebook),
+			   GTK_DEST_DEFAULT_MOTION |
 			   GTK_DEST_DEFAULT_DROP,
-                           url_drag_types, G_N_ELEMENTS (url_drag_types),
-                           GDK_ACTION_MOVE | GDK_ACTION_COPY);
+			   url_drag_types, G_N_ELEMENTS (url_drag_types),
+			   GDK_ACTION_MOVE | GDK_ACTION_COPY);
 
 	notebook->priv->tabs_vis_notifier_id = eel_gconf_notification_add
 		(CONF_ALWAYS_SHOW_TABS_BAR,
@@ -864,32 +550,16 @@ sync_load_status (EphyTab *tab, GParamSpec *pspec, GtkWidget *proxy)
 }
 
 static void
-sync_icon (EphyTab *tab, GParamSpec *pspec, GtkWidget *proxy)
+sync_icon (EphyTab *tab,
+	   GParamSpec *pspec,
+	   GtkWidget *proxy)
 {
-	EphyFaviconCache *cache;
-	GdkPixbuf *pixbuf = NULL;
-	GtkImage *icon = NULL;
-	const char *address;
-
-	cache = EPHY_FAVICON_CACHE
-		(ephy_embed_shell_get_favicon_cache (EPHY_EMBED_SHELL (ephy_shell)));
-	address = ephy_tab_get_icon_address (tab);
-
-	if (address)
-	{
-		pixbuf = ephy_favicon_cache_get (cache, address);
-	}
+	GtkImage *icon;
 
 	icon = GTK_IMAGE (g_object_get_data (G_OBJECT (proxy), "icon"));
-	if (icon)
-	{
-		gtk_image_set_from_pixbuf (icon, pixbuf);
-	}
-
-	if (pixbuf)
-	{
-		g_object_unref (pixbuf);
-	}
+	g_return_if_fail (icon != NULL);
+	
+	gtk_image_set_from_pixbuf (icon, ephy_tab_get_icon (tab));
 }
 
 static void
@@ -917,40 +587,38 @@ sync_label (EphyTab *tab, GParamSpec *pspec, GtkWidget *proxy)
 static void
 close_button_clicked_cb (GtkWidget *widget, GtkWidget *tab)
 {
-	EphyNotebook *notebook;
-	gboolean inhibited = FALSE;
+	GtkWidget *notebook;
 
-	notebook = EPHY_NOTEBOOK (gtk_widget_get_parent (tab));
-	g_signal_emit (G_OBJECT (notebook), signals[TAB_DELETE], 0, tab, &inhibited);
-
-	if (inhibited == FALSE)
-	{
-		ephy_notebook_remove_tab (notebook, EPHY_TAB (tab));
-	}
+	notebook = gtk_widget_get_parent (tab);
+	g_signal_emit (notebook, signals[TAB_CLOSE_REQUEST], 0, tab);
 }
 
 static void
-tab_label_style_set_cb (GtkWidget *label,
+tab_label_style_set_cb (GtkWidget *hbox,
 			GtkStyle *previous_style,
-			GtkWidget *hbox)
+		        gpointer user_data)
 {
 	PangoFontMetrics *metrics;
 	PangoContext *context;
+	GtkWidget *button;
 	int char_width, h, w;
 
-	context = gtk_widget_get_pango_context (label);
+	context = gtk_widget_get_pango_context (hbox);
 	metrics = pango_context_get_metrics (context,
-			                     label->style->font_desc,
+					     hbox->style->font_desc,
 					     pango_context_get_language (context));
 
 	char_width = pango_font_metrics_get_approximate_digit_width (metrics);
 	pango_font_metrics_unref (metrics);
 
-	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (label),
+	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (hbox),
 					   GTK_ICON_SIZE_MENU, &w, &h);
 
 	gtk_widget_set_size_request
 		(hbox, TAB_WIDTH_N_CHARS * PANGO_PIXELS(char_width) + 2 * w, -1);
+
+	button = g_object_get_data (G_OBJECT (hbox), "close-button");
+	gtk_widget_set_size_request (button, w + 2, h + 2);
 }
 
 static GtkWidget *
@@ -958,7 +626,7 @@ build_tab_label (EphyNotebook *nb, EphyTab *tab)
 {
 	GtkWidget *hbox, *label_hbox, *label_ebox;
 	GtkWidget *label, *close_button, *image, *spinner, *icon;
-	GtkIconSize close_icon_size;
+	GtkRcStyle *rcstyle;
 
 	/* set hbox spacing and label padding (see below) so that there's an
 	 * equal amount of space around the label */
@@ -977,18 +645,22 @@ build_tab_label (EphyNotebook *nb, EphyTab *tab)
 			       GTK_RELIEF_NONE);
 	/* don't allow focus on the close button */
 	gtk_button_set_focus_on_click (GTK_BUTTON (close_button), FALSE);
+	gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);
 
-	close_icon_size = gtk_icon_size_from_name (EPHY_ICON_SIZE_TAB_BUTTON);
-	image = gtk_image_new_from_stock (EPHY_STOCK_CLOSE_TAB, close_icon_size);
+	rcstyle = gtk_rc_style_new ();
+	rcstyle->xthickness = rcstyle->ythickness = 0;
+	gtk_widget_modify_style (close_button, rcstyle);
+	gtk_rc_style_unref (rcstyle),
+
+	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
 	gtk_container_add (GTK_CONTAINER (close_button), image);
 	gtk_box_pack_start (GTK_BOX (hbox), close_button, FALSE, FALSE, 0);
 
 	gtk_tooltips_set_tip (nb->priv->title_tips, close_button,
 			      _("Close tab"), NULL);
 
-	g_signal_connect (G_OBJECT (close_button), "clicked",
-                          G_CALLBACK (close_button_clicked_cb),
-                          tab);
+	g_signal_connect (close_button, "clicked",
+			  G_CALLBACK (close_button_clicked_cb), tab);
 
 	/* setup load feedback */
 	spinner = ephy_spinner_new ();
@@ -1000,16 +672,23 @@ build_tab_label (EphyNotebook *nb, EphyTab *tab)
 	gtk_box_pack_start (GTK_BOX (label_hbox), icon, FALSE, FALSE, 0);
 
 	/* setup label */
-        label = gtk_label_new ("");
+	label = gtk_label_new ("");
 	gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
 	gtk_label_set_single_line_mode (GTK_LABEL (label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-        gtk_misc_set_padding (GTK_MISC (label), 0, 0);
+	gtk_misc_set_padding (GTK_MISC (label), 0, 0);
 	gtk_box_pack_start (GTK_BOX (label_hbox), label, TRUE, TRUE, 0);
 
 	/* Set minimal size */
-	g_signal_connect (label, "style-set",
-			  G_CALLBACK (tab_label_style_set_cb), hbox);
+	g_signal_connect (hbox, "style-set",
+			  G_CALLBACK (tab_label_style_set_cb), NULL);
+
+	/* Set up drag-and-drop target */
+	g_signal_connect (hbox, "drag_data_received",
+			  G_CALLBACK (notebook_drag_data_received_cb), tab);
+	gtk_drag_dest_set (hbox, GTK_DEST_DEFAULT_ALL,
+			   url_drag_types, G_N_ELEMENTS (url_drag_types),
+			   GDK_ACTION_MOVE | GDK_ACTION_COPY);
 
 	gtk_widget_show (hbox);
 	gtk_widget_show (label_ebox);
@@ -1025,6 +704,18 @@ build_tab_label (EphyNotebook *nb, EphyTab *tab)
 	g_object_set_data (G_OBJECT (hbox), "close-button", close_button);
 	g_object_set_data (G_OBJECT (hbox), "tooltips", nb->priv->title_tips);
 
+	/* Hook the label up to the tab properties */
+	sync_icon (tab, NULL, hbox);
+	sync_label (tab, NULL, hbox);
+	sync_load_status (tab, NULL, hbox);
+
+	g_signal_connect_object (tab, "notify::icon",
+				 G_CALLBACK (sync_icon), hbox, 0);
+	g_signal_connect_object (tab, "notify::title",
+				 G_CALLBACK (sync_label), hbox, 0);
+	g_signal_connect_object (tab, "notify::load-status",
+				 G_CALLBACK (sync_load_status), hbox, 0);
+
 	return hbox;
 }
 
@@ -1036,67 +727,82 @@ ephy_notebook_set_show_tabs (EphyNotebook *nb, gboolean show_tabs)
 	update_tabs_visibility (nb, FALSE);
 }
 
-void
-ephy_notebook_add_tab (EphyNotebook *nb,
+static int
+ephy_notebook_insert_page (GtkNotebook *gnotebook,
+			   GtkWidget *tab_widget,
+			   GtkWidget *tab_label,
+			   GtkWidget *menu_label,
+			   int position)
+{
+	EphyNotebook *notebook = EPHY_NOTEBOOK (gnotebook);
+
+	/* Destroy passed-in tab label */
+	if (tab_label != NULL)
+	{
+		g_object_ref_sink (tab_label);
+		g_object_unref (tab_label);
+	}
+
+	g_assert (EPHY_IS_TAB (tab_widget));
+
+	tab_label = build_tab_label (notebook, EPHY_TAB (tab_widget));
+
+	update_tabs_visibility (notebook, TRUE);
+
+	position = GTK_NOTEBOOK_CLASS (parent_class)->insert_page (gnotebook,
+								   tab_widget,
+								   tab_label,
+								   menu_label,
+								   position);
+
+	gtk_notebook_set_tab_reorderable (gnotebook, tab_widget, TRUE);
+
+	return position;
+}
+
+int
+ephy_notebook_add_tab (EphyNotebook *notebook,
 		       EphyTab *tab,
 		       int position,
 		       gboolean jump_to)
 {
-	GtkWidget *label;
+	GtkNotebook *gnotebook = GTK_NOTEBOOK (notebook);
 
-	g_return_if_fail (EPHY_IS_TAB (tab));
+	g_return_val_if_fail (EPHY_IS_NOTEBOOK (notebook), -1);
 
-	label = build_tab_label (nb, tab);
-
-	update_tabs_visibility (nb, TRUE);
-
-	gtk_notebook_insert_page (GTK_NOTEBOOK (nb), GTK_WIDGET (tab),
-				  label, position);
-
-	/* Set up drag-and-drop target */
-	g_signal_connect (G_OBJECT(label), "drag_data_received",
-			  G_CALLBACK(notebook_drag_data_received_cb), tab);
-	gtk_drag_dest_set (label, GTK_DEST_DEFAULT_ALL,
-			   url_drag_types, G_N_ELEMENTS (url_drag_types),
-			   GDK_ACTION_MOVE | GDK_ACTION_COPY);
-
-	sync_icon (tab, NULL, label);
-	sync_label (tab, NULL, label);
-	sync_load_status (tab, NULL, label);
-
-	g_signal_connect_object (tab, "notify::icon",
-			         G_CALLBACK (sync_icon), label, 0);
-	g_signal_connect_object (tab, "notify::title",
-			         G_CALLBACK (sync_label), label, 0);
-	g_signal_connect_object (tab, "notify::load-status",
-				 G_CALLBACK (sync_load_status), label, 0);
-
-	g_signal_emit (G_OBJECT (nb), signals[TAB_ADDED], 0, tab);
+	position = gtk_notebook_insert_page (GTK_NOTEBOOK (notebook),
+					     GTK_WIDGET (tab),
+					     NULL,
+					     position);
 
+	/* FIXME gtk bug! */
 	/* The signal handler may have reordered the tabs */
-	position = gtk_notebook_page_num (GTK_NOTEBOOK (nb), GTK_WIDGET (tab));
+	position = gtk_notebook_page_num (gnotebook, GTK_WIDGET (tab));
 
 	if (jump_to)
 	{
-		gtk_notebook_set_current_page (GTK_NOTEBOOK (nb), position);
+		gtk_notebook_set_current_page (gnotebook, position);
 		g_object_set_data (G_OBJECT (tab), "jump_to",
 				   GINT_TO_POINTER (jump_to));
 
 	}
+
+	return position;
 }
 
 static void
-smart_tab_switching_on_closure (EphyNotebook *nb,
-				EphyTab *tab)
+smart_tab_switching_on_closure (EphyNotebook *notebook,
+				GtkWidget *tab)
 {
+	EphyNotebookPrivate *priv = notebook->priv;
 	gboolean jump_to;
 
 	jump_to = GPOINTER_TO_INT (g_object_get_data
 				   (G_OBJECT (tab), "jump_to"));
 
-	if (!jump_to || !nb->priv->focused_pages)
+	if (!jump_to || !priv->focused_pages)
 	{
-		gtk_notebook_next_page (GTK_NOTEBOOK (nb));
+		gtk_notebook_next_page (GTK_NOTEBOOK (notebook));
 	}
 	else
 	{
@@ -1105,65 +811,57 @@ smart_tab_switching_on_closure (EphyNotebook *nb,
 		int page_num;
 
 		/* activate the last focused tab */
-		l = g_list_last (nb->priv->focused_pages);
+		l = g_list_last (priv->focused_pages);
 		child = GTK_WIDGET (l->data);
-		page_num = gtk_notebook_page_num (GTK_NOTEBOOK (nb),
+		page_num = gtk_notebook_page_num (GTK_NOTEBOOK (notebook),
 						  child);
-		gtk_notebook_set_current_page
-			(GTK_NOTEBOOK (nb), page_num);
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook),
+					       page_num);
 	}
 }
 
-void
-ephy_notebook_remove_tab (EphyNotebook *nb,
-			  EphyTab *tab)
+static void
+ephy_notebook_remove (GtkContainer *container,
+		      GtkWidget *tab_widget)
 {
+	GtkNotebook *gnotebook = GTK_NOTEBOOK (container);
+	EphyNotebook *notebook = EPHY_NOTEBOOK (container);
+	EphyNotebookPrivate *priv = notebook->priv;
+	GtkWidget *tab_label, *ebox;
 	int position, curr;
-	GtkWidget *label, *ebox;
 
-	g_return_if_fail (EPHY_IS_NOTEBOOK (nb));
-	g_return_if_fail (EPHY_IS_TAB (tab));
+	g_assert (EPHY_IS_TAB (tab_widget));
 
 	/* Remove the page from the focused pages list */
-	nb->priv->focused_pages =  g_list_remove (nb->priv->focused_pages,
-						  tab);
+	priv->focused_pages =  g_list_remove (priv->focused_pages, tab_widget);
 
-	position = gtk_notebook_page_num (GTK_NOTEBOOK (nb), GTK_WIDGET (tab));
-	curr = gtk_notebook_get_current_page (GTK_NOTEBOOK (nb));
+	position = gtk_notebook_page_num (gnotebook, tab_widget);
+	curr = gtk_notebook_get_current_page (gnotebook);
 
 	if (position == curr)
 	{
-		smart_tab_switching_on_closure (nb, tab);
+		smart_tab_switching_on_closure (notebook, tab_widget);
 	}
 
-	label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb), GTK_WIDGET (tab));
-	ebox = GTK_WIDGET (g_object_get_data (G_OBJECT (label), "label-ebox"));
-	gtk_tooltips_set_tip (GTK_TOOLTIPS (nb->priv->title_tips), ebox, NULL, NULL);
+	/* Prepare tab label for destruction */
+	tab_label = gtk_notebook_get_tab_label (gnotebook, tab_widget);
+	ebox = GTK_WIDGET (g_object_get_data (G_OBJECT (tab_label), "label-ebox"));
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (priv->title_tips), ebox, NULL, NULL);
 
-	g_signal_handlers_disconnect_by_func (tab,
-					      G_CALLBACK (sync_icon), label);
-	g_signal_handlers_disconnect_by_func (tab,
-					      G_CALLBACK (sync_label), label);
-	g_signal_handlers_disconnect_by_func (tab,
-					      G_CALLBACK (sync_load_status), label);
+	g_signal_handlers_disconnect_by_func
+		(tab_widget, G_CALLBACK (sync_icon), tab_label);
+	g_signal_handlers_disconnect_by_func
+		(tab_widget, G_CALLBACK (sync_label), tab_label);
+	g_signal_handlers_disconnect_by_func
+		(tab_widget, G_CALLBACK (sync_load_status), tab_label);
 
-	/**
-	 * we ref the tab so that it's still alive while the tabs_removed
-	 * signal is processed.
-	 */
-	g_object_ref (tab);
+	GTK_CONTAINER_CLASS (parent_class)->remove (container, tab_widget);
 
-	gtk_notebook_remove_page (GTK_NOTEBOOK (nb), position);
-
-	update_tabs_visibility (nb, FALSE);
-
-	g_signal_emit (G_OBJECT (nb), signals[TAB_REMOVED], 0, tab);
-
-	g_object_unref (tab);
+	update_tabs_visibility (notebook, FALSE);
 
 	/* if that was the last tab, destroy the window */
-	if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (nb)) == 0)
+	if (gtk_notebook_get_n_pages (gnotebook) == 0)
 	{
-		gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (nb)));
+		gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (notebook)));
 	}
 }
