@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tabs-menu.c,v 1.39 2005/11/08 12:46:43 chpe Exp $
+ *  $Id: ephy-tabs-menu.c,v 1.41 2006/03/23 16:43:28 chpe Exp $
  */
 
 #include "config.h"
@@ -130,9 +130,10 @@ sync_tab_title (EphyTab *tab,
 }
 
 static void
-tab_added_cb (EphyNotebook *notebook,
-	      EphyTab *tab,
-	      EphyTabsMenu *menu)
+notebook_page_added_cb (EphyNotebook *notebook,
+			EphyTab *tab,
+			guint position,
+			EphyTabsMenu *menu)
 {
 	EphyTabsMenuPrivate *priv = menu->priv;
 	GtkAction *action;
@@ -176,9 +177,10 @@ tab_added_cb (EphyNotebook *notebook,
 }
 
 static void
-tab_removed_cb (EphyNotebook *notebook,
-		EphyTab *tab,
-		EphyTabsMenu *menu)
+notebook_page_removed_cb (EphyNotebook *notebook,
+			  EphyTab *tab,
+			  guint position,
+			  EphyTabsMenu *menu)
 {
 	EphyTabsMenuPrivate *priv = menu->priv;
 	GtkAction *action;
@@ -201,8 +203,10 @@ tab_removed_cb (EphyNotebook *notebook,
 }
 
 static void
-tabs_reordered_cb (EphyNotebook *notebook,
-		   EphyTabsMenu *menu)
+notebook_page_reordered_cb (EphyNotebook *notebook,
+			    EphyTab *tab,
+			    guint position,
+			    EphyTabsMenu *menu)
 {
 	LOG ("tabs_reordered_cb");
  
@@ -278,12 +282,12 @@ ephy_tabs_menu_set_window (EphyTabsMenu *menu,
 			  G_CALLBACK (sync_active_tab), menu);
 
 	notebook = ephy_window_get_notebook (window);
-	g_signal_connect_object (notebook, "tab_added",
-				 G_CALLBACK (tab_added_cb), menu, 0);
-	g_signal_connect_object (notebook, "tab_removed",
-				 G_CALLBACK (tab_removed_cb), menu, 0);
-	g_signal_connect_object (notebook, "tabs_reordered",
-				 G_CALLBACK (tabs_reordered_cb), menu, 0);
+	g_signal_connect_object (notebook, "page-added",
+				 G_CALLBACK (notebook_page_added_cb), menu, 0);
+	g_signal_connect_object (notebook, "page-removed",
+				 G_CALLBACK (notebook_page_removed_cb), menu, 0);
+	g_signal_connect_object (notebook, "page-reordered",
+				 G_CALLBACK (notebook_page_reordered_cb), menu, 0);
 }
 
 static void
@@ -385,7 +389,7 @@ tab_set_action_accelerator (GtkActionGroup *action_group,
 		accel_key = 0;
 		accel_number = (tab_number + 1) % 10;
 
-		g_snprintf (accel, 7, "<alt>%d", accel_number);
+		g_snprintf (accel, sizeof (accel), "<alt>%d", accel_number);
 
 		gtk_accelerator_parse (accel, &accel_key, &accel_mods);
 
