@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: window-commands.c,v 1.182 2006/01/16 23:51:41 chpe Exp $
+ *  $Id: window-commands.c,v 1.192 2006/03/23 16:43:28 chpe Exp $
  */
 
 #include "config.h"
@@ -45,6 +45,7 @@
 #include "ephy-find-toolbar.h"
 #include "ephy-location-entry.h"
 #include "ephy-bookmarks-ui.h"
+#include "ephy-link.h"
 #include "pdm-dialog.h"
 
 #include <string.h>
@@ -240,7 +241,7 @@ window_cmd_file_bookmark_page (GtkAction *action,
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	ephy_bookmarks_ui_add_bookmark (GTK_WIDGET (window),
+	ephy_bookmarks_ui_add_bookmark (GTK_WINDOW (window),
 					ephy_tab_get_address (tab),
 					ephy_tab_get_title (tab));
 }
@@ -331,18 +332,21 @@ void
 window_cmd_file_close_window (GtkAction *action,
 		              EphyWindow *window)
 {
-	EphyEmbed *embed;
+	GtkWidget *notebook;
+	EphyTab *tab;
+
+	notebook = ephy_window_get_notebook (window);
 
 	if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_QUIT) &&
-	    gtk_notebook_get_n_pages (GTK_NOTEBOOK (ephy_window_get_notebook (window))) == 1)
+	    gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook)) == 1)
 	{
 		return;
 	}
 
-	embed = ephy_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
+	tab = ephy_window_get_active_tab (window);
+	g_return_if_fail (tab != NULL);
 
-	ephy_embed_close (embed);
+	g_signal_emit_by_name (notebook, "tab-close-request", tab);
 }
 
 void
@@ -721,6 +725,7 @@ window_cmd_help_about (GtkAction *action,
 	};
 	const char * const documenters[] = {
 		"Piers Cornwell", 
+		"Victor Osadci",
 		"Patanjali Somayaji",
 		"David Bordoley",
 		"",
@@ -729,12 +734,43 @@ window_cmd_help_about (GtkAction *action,
 		NULL
 	};
 
+	const char *licence_part[] = {
+		N_("The GNOME Web Browser is free software; you can redistribute it and/or modify "
+		   "it under the terms of the GNU General Public License as published by "
+		   "the Free Software Foundation; either version 2 of the License, or "
+		   "(at your option) any later version."),
+		N_("The GNOME Web Browser is distributed in the hope that it will be useful, "
+		   "but WITHOUT ANY WARRANTY; without even the implied warranty of "
+		   "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
+		   "GNU General Public License for more details."),
+		N_("You should have received a copy of the GNU General Public License "
+		   "along with the GNOME Web Browser; if not, write to the Free Software Foundation, Inc., "
+		   "59 Temple Place, Suite 330, Boston, MA  02111-1307  USA")
+	};
+
+	EphyEmbedShell *shell;
+	EphyEmbedSingle *single;
+	char *licence, *comments;
+
+	shell = ephy_embed_shell_get_default ();
+	single = EPHY_EMBED_SINGLE (ephy_embed_shell_get_embed_single (shell));
+
+	comments = g_strdup_printf (_("Lets you view web pages and find information on the internet.\n"
+				      "Powered by Gecko %s"),
+				      ephy_embed_single_get_backend_name (single) + strlen ("gecko-"));
+	licence = g_strjoin ("\n\n",
+			     _(licence_part[0]),
+			     _(licence_part[1]),
+			     _(licence_part[2]),
+			    NULL);
+
 	gtk_show_about_dialog (GTK_WINDOW (window),
 			       "name", _("GNOME Web Browser"),
 			       "version", VERSION,
 			       "copyright", "Copyright © 2002-2004 Marco Pesenti Gritti\n"
 					    "Copyright © 2003-2006 The GNOME Web Browser Developers",
 			       "authors", authors,
+			       "comments", comments,
 			       "documenters", documenters,
 				/* Translators: This is a special message that shouldn't be translated
 				 * literally. It is used in the about box to give credits to
@@ -748,7 +784,12 @@ window_cmd_help_about (GtkAction *action,
 			       "logo-icon-name", "web-browser",
 			       "website", "http://www.gnome.org/projects/epiphany",
 			       "website-label", _("GNOME Web Browser Website"),
+			       "license", licence,
+			       "wrap-license", TRUE,
 			       NULL);
+
+	g_free (comments);
+	g_free (licence);
 }
 
 void
@@ -793,61 +834,32 @@ void
 window_cmd_tabs_move_left  (GtkAction *action,
 			    EphyWindow *window)
 {
-	GtkWidget *notebook;
+	GtkWidget *child;
+	GtkNotebook *notebook;
 	int page;
 
-	notebook = ephy_window_get_notebook (window);
-	page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+	notebook = GTK_NOTEBOOK (ephy_window_get_notebook (window));
+	page = gtk_notebook_get_current_page (notebook);
+	if (page < 1) return;
 
-	if (page > 0)
-	{
-		GtkWidget *child;
-
-		child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), page);
-		ephy_notebook_move_tab (EPHY_NOTEBOOK (notebook), NULL,
-					EPHY_TAB (child), page - 1);
-	}
+	child = gtk_notebook_get_nth_page (notebook, page);
+	gtk_notebook_reorder_child (notebook, child, page - 1);
 }
 
 void window_cmd_tabs_move_right (GtkAction *action,
 				 EphyWindow *window)
 {
-	GtkWidget *notebook;
-	int page;
-	int last_page;
+	GtkWidget *child;
+	GtkNotebook *notebook;
+	int page, n_pages;
 
-	notebook = ephy_window_get_notebook (window);
-	page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
-	last_page = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook)) - 1;
+	notebook = GTK_NOTEBOOK (ephy_window_get_notebook (window));
+	page = gtk_notebook_get_current_page (notebook);
+	n_pages = gtk_notebook_get_n_pages (notebook) - 1;
+	if (page > n_pages - 1) return;
 
-	if (page != last_page)
-	{
-		GtkWidget *child;
-
-		child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), page);
-		ephy_notebook_move_tab (EPHY_NOTEBOOK (notebook), NULL,
-					EPHY_TAB (child), page + 1);
-	}
-}
-
-void
-window_cmd_tabs_detach  (GtkAction *action,
-			 EphyWindow *window)
-{
-	EphyTab *tab;
-	GtkWidget *nb;
-	EphyWindow *new_win;
-
-	nb = ephy_window_get_notebook (window);
-	if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (nb)) <= 1) return;
-
-	tab = ephy_window_get_active_tab (window);
-
-	new_win = ephy_window_new ();
-	ephy_notebook_move_tab (EPHY_NOTEBOOK (ephy_window_get_notebook (window)),
-				EPHY_NOTEBOOK (ephy_window_get_notebook (new_win)),
-				tab, 0);
-	gtk_widget_show (GTK_WIDGET (new_win));
+	child = gtk_notebook_get_nth_page (notebook, page);
+	gtk_notebook_reorder_child (notebook, child, page + 1);
 }
 
 void
@@ -862,7 +874,9 @@ window_cmd_load_location (GtkAction *action,
 
 	if (location)
 	{
-		ephy_window_load_url (window, location);
+		ephy_link_open (EPHY_LINK (window), location,
+			        ephy_window_get_active_tab (window),
+				ephy_link_flags_from_current_event ());
 	}
 }
 
